'*********************************************************************************
'*  OpenGL "glu.inc" for PowerBASIC-Windows 7.0x                                 *
'*  Port Version 1.0d (February 03, 2003)                                        *
'*                                                                               *
'*  by Scott Martindale (smartndl@hotmail.com)                                   *
'*                                                                               *
'*  LEGAL:                                                                       *
'*  You have a royalty-free right to use, modify, reproduce and distribute this  *
'*  file (and/or any modified version) in any way you find useful, provided that *
'*  you agree that PowerBASIC, Silicon Graphics, present or future contributors  *
'*  have no warranty, obligation or liability for its contents.                  *
'*                                                                               *
'*********************************************************************************
'*  PowerBASIC GLU (GL-Utility) Macros:                                          *
'*                                                                               *
'*  These macros initially are written for testing, and debugging. After full    *
'*  testing, many of the heavily used macros will be optimized, or written in    *
'*  in-line assembler.                                                           *
'*                                                                               *
'*  Included GLU-Functions (Macros):                                             *
'*                                   "gluOrtho2D"                                *
'*                                   "gluLookAt"                                 *
'*                                   "gluPerspective"                            *
'*                                   "gluPickMatrix"                             *
'*                                                                               *
'*********************************************************************************
'*  METASTATEMENTS:                                                              *
'*********************************************************************************
    #IF NOT %DEF(%PB_GLinc)
        #INCLUDE "gl.inc"
        %PB_GLinc = 1
    #ENDIF

    #IF NOT %DEF(%PB_GLUinc)
    %PB_GLUinc = 1

'*********************************************************************************
'* GLU-Functions (Macros):                                                       *
'*********************************************************************************
' gluOrtho2D
    MACRO gluOrtho2D(nLeft, nRight, nBottom, nTop)
        glOrtho(nLeft, nRight, nBottom, nTop, -1#, 1#)
    END MACRO

' gluLookAt
    MACRO gluLookAt(eyex, eyey, eyez, centerx, centery, centerz, upx, upy, upz)
        MACROTEMP m, x, y, z, mag
        DIM m(15) AS DOUBLE, x(2) AS DOUBLE, y(2) AS DOUBLE, _
        z(2) AS DOUBLE, mag AS DOUBLE

        z(0) = eyex - centerx
        z(1) = eyey - centery
        z(2) = eyez - centerz

        mag = SQR((z(0)*z(0)) + (z(1)*z(1)) + (z(2)*z(2)))
        IF mag THEN
           z(0) = z(0)/mag
           z(1) = z(1)/mag
           z(2) = z(2)/mag
        END IF

        x(0) =  (upy  * z(2)) - (upz  * z(1))
        x(1) = -(upx  * z(2)) + (upz  * z(0))
        x(2) =  (upx  * z(1)) - (upy  * z(0))
        y(0) =  (z(1) * x(2)) - (z(2) * x(1))
        y(1) = -(z(0) * x(2)) + (z(2) * x(0))
        y(2) =  (z(0) * x(1)) - (z(1) * x(0))

        mag  =  SQR((x(0)*x(0)) + (x(1)*x(1)) + (x(2)*x(2)))
        IF mag THEN
           x(0) = x(0)/mag
           x(1) = x(1)/mag
           x(2) = x(2)/mag
        END IF

        mag = SQR((y(0)*y(0)) + (y(1)*y(1)) + (y(2)*y(2)))
        IF mag THEN
           y(0) = y(0)/mag
           y(1) = y(1)/mag
           y(2) = y(2)/mag
        END IF

        m(00) = x(0) : m(01) = y(0) : m(02) = z(0) : m(03) = 0.0#
        m(04) = x(1) : m(05) = y(1) : m(06) = z(1) : m(07) = 0.0#
        m(08) = x(2) : m(09) = y(2) : m(10) = z(2) : m(11) = 0.0#
        m(12) = 0.0# : m(13) = 0.0# : m(14) = 0.0# : m(15) = 1.0#

        glMultMatrixd(m)
        glTranslated (-eyex, -eyey, -eyez)
    END MACRO

' gluPerspective
    MACRO gluPerspective(fovy, nAspect, zNear, zFar)
        MACROTEMP xmin, xmax, ymin, ymax
        DIM xmin AS DOUBLE : DIM xmax AS DOUBLE
        DIM ymin AS DOUBLE : DIM ymax AS DOUBLE
        ymax = zNear * TAN((fovy * 3.14159265358979#)/360#)
        ymin = -ymax
        xmin = ymin * nAspect
        xmax = ymax * nAspect
        glFrustum xmin, xmax, ymin, ymax, zNear, zFar
    END MACRO

' gluPickMatrix
    MACRO gluPickMatrix(x, y, nwidth, nheight, viewport)
        MACROTEMP m
        DIM m(15) AS SINGLE
        m(00) = viewport(2) / nwidth   'Scale-X
        m(05) = viewport(3) / nheight  'Scale-Y
        m(10) = 1!                     'Identity major-diagonal filler
        m(12) = (viewport(2) + 2.0 * (viewport(0) - x)) / nwidth  'Trans-X
        m(13) = (viewport(3) + 2.0 * (viewport(1) - y)) / nheight 'Trans-Y
        m(15) = 1!                     'Identity major-diagonal filler
        glMultMatrixf(m)
    END MACRO

#ENDIF
'*********************************************************************************